/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "quakedef.h"
#include "snd_internal.h"
#include <SDL/SDL.h>

extern int snd_freq;

/*
==============
S_LoadSound
==============
*/
sfxcache_t *S_LoadSound (sfx_t *s)
{
    std::string filename;
    byte *data;
    sfxcache_t *sc;
    SDL_RWops *rw;
    SDL_AudioSpec wav_spec;
    Uint8 *wav_audio_buffer;
    Uint32 wav_audio_size;
    SDL_AudioCVT converter;
    Uint32 final_size;

// see if still in memory
    sc = (sfxcache_t *)Cache_Check (&s->cache);
    if (sc)
        return sc;

// load it in
    filename = "sound/";
    filename += s->name;

    File fp;
    if (!fp.OpenRead(filename))
    {
        Con_Printf ("Couldn't open %s\n", filename.c_str());
        return NULL;
    }
    
    data = fp.Read();
    if (!data)
    {
        Con_Printf ("Couldn't read %s\n", filename.c_str());
        return NULL;
    }
    
    rw = SDL_RWFromConstMem(data, fp.Size());
    if (!rw)
    {
        Con_Printf ("Can't get SDL RW for %s\n", filename.c_str());
        return NULL;
    }
    
    fp.Close();

    if (!SDL_LoadWAV_RW(rw, 0, &wav_spec, &wav_audio_buffer, &wav_audio_size))
    {
        Con_Printf ("Can't get SDL to load WAV for %s\n", filename.c_str());
        return NULL;
    }

    if (SDL_BuildAudioCVT(&converter, wav_spec.format, wav_spec.channels,
        wav_spec.freq, AUDIO_S16SYS, 2, snd_freq) == -1)
    {
        Con_Printf ("Can't build conversion filter for %s\n", filename.c_str());
        SDL_FreeWAV(wav_audio_buffer);
        return NULL;
    }

    converter.buf = (Uint8 *)malloc(wav_audio_size * converter.len_mult);
    converter.len = wav_audio_size;
    memcpy(converter.buf, wav_audio_buffer, wav_audio_size);

    SDL_FreeWAV(wav_audio_buffer);

    if (SDL_ConvertAudio(&converter) != 0)
    {
        Con_Printf("Couldn't convert %s\n", filename.c_str());
        free(converter.buf);
        return NULL;
    }

    final_size = wav_audio_size * converter.len_mult;

    sc = (sfxcache_t *)Cache_Alloc (&s->cache, final_size + sizeof(sfxcache_t), s->name);
    if (!sc)
        return NULL;

    sc->length = final_size / 4;
    memcpy(sc->data, converter.buf, final_size);

    sc->looppos = -1;

    if (fp.OpenRead("sound.loops"))
    {
        while (!fp.eof())
        {
            std::string buf = fp.GetLine();
            if (buf.compare(0, filename.length(), filename) != 0)
                continue;
            
            sc->looppos = Util::lexical_cast<int>(buf.substr(filename.length() + 1));
        }
    }

    return sc;
}

