/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "quakedef.h"
#include "snd_internal.h"

typedef struct
{
    int left;
    int right;
} portable_samplepair_t;

portable_samplepair_t *paintbuffer;

void S_InitPaintBuffer(size_t len)
{
    paintbuffer = (portable_samplepair_t *)malloc(sizeof(portable_samplepair_t) * len);
}

void S_ShutdownPaintBuffer(void)
{
    free(paintbuffer);
}


void S_TransferPaintBuffer(unsigned char *stream, int num_samples)
{
    int i, val;
    int snd_vol = Cvar_VariableValue("volume") * 256;
    short *snd_out = (short *)stream;

    for (i=0 ; i<num_samples ; i++)
    {
        val = (paintbuffer[i].left * snd_vol)>>8;
        *snd_out = (val > 0x7fff ? 0x7fff : (val < (short)0x8000 ? (short)0x8000 : val));
        ++snd_out;

        val = (paintbuffer[i].right * snd_vol)>>8;
        *snd_out = (val > 0x7fff ? 0x7fff : (val < (short)0x8000 ? (short)0x8000 : val));
        ++snd_out;
    }
}


void S_MixSDLAudio (void *unused, unsigned char *stream, int len)
{
    int i, j;
    channel_t *ch;
    sfxcache_t *sc;
    int offset, samps_left, chan_left, count;
    int num_samples;
    short *sfx_data;

    num_samples = len / 4;

    // clear the paint buffer
    memset(paintbuffer, 0, num_samples * sizeof(portable_samplepair_t));

    // paint in the channels
    ch = channels;
    for (i=0; i<total_channels ; i++, ch++)
    {
        if (!ch->sfx)
            continue;
        if (!ch->leftvol && !ch->rightvol)
            continue;
        if (!ch->sfx->cache.data)
            continue;
        sc = (sfxcache_t *)ch->sfx->cache.data;

        offset = 0;
        while (offset < num_samples)
        {
            samps_left = num_samples - offset;
            chan_left = sc->length - ch->pos;

            if (samps_left > chan_left)
                count = chan_left;
            else
                count = samps_left;

            if (count > 0)
            {
                sfx_data = (short *)sc->data + (ch->pos * 2);

                for (j = 0 ; j < count ; j++)
                {
                    paintbuffer[j + offset].left += (sfx_data[j*2] * ch->leftvol) >> 8;
                    paintbuffer[j + offset].right += (sfx_data[j*2 + 1] * ch->rightvol) >> 8;
                }

                offset += count;
                ch->pos += count;
            }

            // if at end of loop, restart
            if (ch->pos >= sc->length)
            {
                if (ch->looping)
                {
                    if (sc->looppos != -1)
                        ch->pos = sc->looppos;
                    else
                        ch->pos = 0;
                }
                else
                {
                    // channel just stopped
                    ch->sfx = NULL;
                    ch->pos = 999999;
                    break;
                }
            }
        }
    }

    // copy to the output stream
    S_TransferPaintBuffer(stream, num_samples);
}

